package org.example.sweater.controller;

import org.example.sweater.domain.Message;
import org.example.sweater.domain.User;
import org.example.sweater.repos.MessagesRepository;
import org.example.sweater.service.FileService;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Map;
import java.util.Objects;

@Controller
public class MainController {
    public MainController(MessagesRepository messagesRepository, FileService fileService) {
        this.messagesRepository = messagesRepository;
        this.fileService = fileService;
    }

    private final MessagesRepository messagesRepository;
    private final FileService fileService;

    @GetMapping("/")
    public String mainPage() {
        return "main";
    }

    @GetMapping("/feed")
    public String feed(
            @RequestParam(required = false)
            String tag,
            @RequestParam(required = false)
            String prefix,
            Model model
    ) {
        Iterable<Message> messages;

        if (prefix != null && !prefix.isEmpty()) {
            messages = messagesRepository.findByTextStartingWith(prefix);
            model.addAttribute("search", prefix);
        } else if (tag != null && !tag.isEmpty()) {
            messages = messagesRepository.findByTag(tag);
            model.addAttribute("tag", tag);
        } else {
            messages = messagesRepository.findAll();
        }

        model.addAttribute("messages", messages);

        return "feed";
    }

    @PostMapping("/feed")
    public String addMessage(
            @AuthenticationPrincipal User user,
            @RequestParam String text,
            @RequestParam String tag,
            @RequestParam("file") MultipartFile file,
            Map<String, Object> model
    ) throws IOException {
        Message createdMessage = new Message(text, tag, user);

        if (file != null && !Objects.requireNonNull(file.getOriginalFilename()).isEmpty()) {
            String savedFileName = fileService.saveFile(file);

            createdMessage.setFilename(savedFileName);
        }

        messagesRepository.save(createdMessage);

        Iterable<Message> allMessages = messagesRepository.findAll();
        model.put("messages", allMessages);

        return "feed";
    }

    @PostMapping("/feed/delete")
    public String removeMessageById(
            @RequestParam Message message,
            Map<String, Object> model
    ) {
        messagesRepository.deleteById(message.getId());
        String fileName = message.getFilename();

        if (fileName != null) {
            fileService.deleteFile(fileName);
        }

        Iterable<Message> foundMessages = messagesRepository.findAll();

        model.put("messages", foundMessages);

        return "feed";
    }
}
